#ifndef _WORLD2D_H
#define _WORLD2D_H

#include <fstream.h>
#include <vector>
#include <string>
//<apstring.h>

#include <CMUgraphics2.h>
#define CMUG1

#define USE_C_ARRAYS

#include "util.h"

class world2d
{
	protected:
		#ifdef USE_C_ARRAYS
			sprite** world;
			spritePic* sprites;
		#else
			smatrix world;
			std::vector<spritePic> sprites;
		#endif

		int worldXSize, worldYSize;
		int spritesSize;

		event quitEvent;
		event moveUpEvent;
		event moveDownEvent;
		event moveLeftEvent;
		event moveRightEvent;
		event jumpEvent;

		int defMoveSpeed;
		int defJumpHeight;

		int spriteWidth;
		int spriteHeight;

		coord2d pos;
		char key;

		window* win;
		bool myWin;

		image character;
		status charStatus;

		void loadWorld(ifstream& fin);
		void loadSprites(ifstream& fin);

		void syncPointers();

		

		image* bkgd;

		short jumpStatus;

	public:
		void showTxtWorld();

		void drawPage(const std::string& str);

		world2d(const std::string& srcFile, const std::string& characterIcon,
			const std::string& title, const int winWidth, const int winHeight,
			const int spritewidth, const int spriteheight);

		world2d(const std::string& srcFile, const std::string& characterIcon,
			const std::string& title, window* wind,
			const int spritewidth, const int spriteheight);

		~world2d();

		inline char& getLastKeyPress() { return key; }
		inline coord2d& getCharacterXYPos() { return pos; }

		inline coord2d coordOfSprite(const coord2d& coord, const coord2d& loc);
		inline unsigned short indexOfSprite(const coord2d& coord);

		inline unsigned short findSpriteIndex(const char type);

		inline status& getStatus() { return charStatus; }

		inline event& getQuitEvent() { return quitEvent; }
		inline event& getMoveUpEvent() { return moveUpEvent; }
		inline event& getMoveDownEvent() { return moveDownEvent; }
		inline event& getMoveLeftEvent() { return moveLeftEvent; }
		inline event& getMoveRightEvent() { return moveRightEvent; }

		inline int& getDefMoveSpeed() { return defMoveSpeed; }

		inline void addBkgdImage(const std::string fileName)
			{	bkgd = new image(fileName, imgType(fileName));	}

		inline bool run();

	private:
		inline void eventScan(coord2d& tmpPos);
		inline void manageJump();
		inline void checkBlocks(const coord2d& tmpPos);
		inline void drawUsingCMUG1();

};

#include "world2d_constructors_destructors.cpp"
#include "world2d.cpp"
#include "world2d_runFunctions.cpp"


#endif	// world2d.h